#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from pathlib import Path
from snakemake.logging import logger
from snakemake.utils import validate
from snakemake.io import load_configfile
import yaml
from collections import OrderedDict
import pprint
import re
import culebrONT
import click
from culebrONT.global_variable import *
from culebrONT.usefull_function import get_install_mode


def get_dir(path):
    """List of directory included on folder"""
    return [elm.name for elm in Path(path).glob("*") if elm.is_dir()]


def get_files_ext(path, extensions, add_ext=True):
    """List of files with specify extension included on folder

    Arguments:
        path (str): a path to folder
        extensions (list or tuple): a list or tuple of extension like (".py")
        add_ext (bool): if True (default), file have extension

    Returns:
        :class:`list`: List of files name with or without extension , with specify extension include on folder
        :class:`list`: List of  all extension found

    Examples:
        >>> version = get_version("/path/to/install/culebront")
        >>> print(version)
            1.3.0
     """
    if not (extensions, (list, tuple)) or not extensions:
        raise ValueError(f'ERROR CulebrONT: "extensions" must be a list or tuple not "{type(extensions)}"')
    tmp_all_files = []
    all_files = []
    files_ext = []
    for ext in extensions:
        tmp_all_files.extend(Path(path).glob(f"**/*{ext}"))

    for elm in tmp_all_files:
        ext = "".join(elm.suffixes)
        if ext not in files_ext:
            files_ext.append(ext)
        if add_ext:
            all_files.append(elm.as_posix())
        else:
            if len(elm.suffixes) > 1:

                all_files.append(Path(elm.stem).stem)
            else:
                all_files.append(elm.stem)
    return all_files, files_ext


def convert_genome_size(size):
    mult = dict(K=10 ** 3, M=10 ** 6, G=10 ** 9, T=10 ** 12, N=1)
    search = re.search(r'^(\d+\.?\d*)\s*(.*)$', size)
    if not search or len(search.groups()) != 2:
        raise ValueError(
            f"CONFIG FILE CHECKING FAIL : not able to convert genome size please only use int value with{' '.join(mult.keys())} upper or lower, N or empty is bp size")
    else:
        value, unit = search.groups()
        if not unit:
            return int(value)
        elif unit and unit.upper() not in mult.keys():
            raise ValueError(
                f"CONFIG FILE CHECKING FAIL : '{unit}' unit value not allow or not able to convert genome size please only use int value with{' '.join(mult.keys())} upper or lower, N or empty is bp size")
        else:
            return int(float(value) * mult[unit.upper()])


class CulebrONT(object):
    """
    to read file config
    """

    def __init__(self, workflow, config):

        # workflow is available only in __init__
        self.snakefile = CULEBRONT_SNAKEFILE
        self.tools_config = None

        if not workflow.overwrite_configfiles:
            raise ValueError("ERROR CulebrONT: You need to use --configfile option to snakemake command line")
        else:
            self.path_config = workflow.overwrite_configfiles[0]
       
        if not workflow.overwrite_clusterconfig and get_install_mode() == "cluster":
            self.cluster_config = load_configfile(CULEBRONT_PROFILE.joinpath("cluster_config.yaml"))
        elif not workflow.overwrite_clusterconfig and get_install_mode() == "local":
            self.cluster_config = None
        else:
            self.cluster_config = workflow.overwrite_clusterconfig

        self.load_tool_configfile()
        # print("\n".join(list(workflow.__dict__.keys())))
        # print(workflow.__dict__)

        # --- Verification Configuration Files --- #
        self.config = config
        self.assembly_tools_activated = []
        self.polishing_tools_activated = []
        self.correction_tools_activated = []
        self.quality_tools_activated = []
        self.quality_step = []
        self.last_steps_list = []
        self.pipeline_stop = None

        self.fastq_files_list = []
        self.fastq_files_ext = []
        self.fastq_gzip = None

        self.illumina_files_list = []
        self.illumina_files_ext = []
        self.illumina_gzip = None

        self.R1 = ""
        self.R2 = ""

        self.add_circular_name = None
        self.TMP = None
        self.TCM = None
        self.TAG = None

        self.draft_to_correction = None
        self.draft_to_correction_index_fai = None
        self.draft_to_correction_index_mmi = None

        self.nb_racon_rounds = None
        self.nb_pilon_rounds = None

        self.use_env_modules = workflow.use_env_modules
        self.use_conda = workflow.use_conda
        self.use_singularity = workflow.use_singularity

        self.__check_config_dic()
        self.__cleaning_for_rerun()

        # With good config to output:
        self.write_config(f"{self.config['DATA']['OUTPUT']}/config_corrected.yaml")

        # using schemas to check mandatory value from yaml format
        try:
            validate(self.config, culebrONT.CULEBRONT_PATH.joinpath("schemas/config.schema.yaml").resolve().as_posix())
        except Exception as e:
            raise ValueError(
                f"{e}\n\nCONFIG FILE CHECKING STRUCTURE FAIL : you need to verify {self.path_config} KEYS:VALUES: {str(e)[30:76]}\n")

    def load_tool_configfile(self):
        if CULEBRONT_USER_TOOLS_PATH.exists() and not CULEBRONT_ARGS_TOOLS_PATH.exists():
            self.tools_config = load_configfile(CULEBRONT_USER_TOOLS_PATH)
        elif CULEBRONT_ARGS_TOOLS_PATH.exists():
            self.tools_config = load_configfile(CULEBRONT_ARGS_TOOLS_PATH)
            CULEBRONT_ARGS_TOOLS_PATH.unlink()
        else:
            self.tools_config = load_configfile(CULEBRONT_TOOLS_PATH)

    def __split_illumina(self):
        R1 = []
        R2 = []
        for fastq in self.illumina_files_list:
            if '_R1' in fastq:
                R1.append(fastq)
            elif '_R2' in fastq:
                R2.append(fastq)
        return R1, R2

    def get_config_value(self, section, key, subsection=None, type_value=None):
        # TODO add type_value to load a good type
        if subsection:
            return self.config[section][subsection][key]
        else:
            return self.config[section][key]

    def set_config_value(self, section, key, value, subsection=None):
        if subsection:
            self.config[section][subsection][key] = value
        else:
            self.config[section][key] = value

    def write_config(self, path):
        p = Path(path).parent
        p.mkdir(parents=True, exist_ok=True)
        with open(path, "w") as config_open:
            config_open.write(self.export_use_yaml)

    @property
    def export_use_yaml(self):
        """Use to print a dump config.yaml with corrected parameters"""

        def represent_dictionary_order(self, dict_data):
            return self.represent_mapping('tag:yaml.org,2002:map', dict_data.items())

        def setup_yaml():
            yaml.add_representer(OrderedDict, represent_dictionary_order)

        setup_yaml()
        return yaml.dump(self.config, default_flow_style=False, sort_keys=False, indent=4)

    def __cleaning_for_rerun(self):
        """Cleaning Report and dags if we rerun the snakemake a second time
        TODO: improve check if final report run maybe check html file"""
        if Path(self.config["DATA"]["OUTPUT"]).joinpath("FINAL_REPORT/CulebrONT_report.html").exists():
            logger.warning(
                f"\nWARNING: If you want to rerun culebront a second time please delete the {self.config['DATA']['OUTPUT']}FINAL_REPORT and REPORT repertories for each sample\n")

    def __check_dir(self, section, key, mandatory=[], subsection=None):
        """Check if path is a directory if not empty
            resolve path on config

        Arguments:
            section (str): the first level on config.yaml
            key (str): the final level on config.yaml
            mandatory (list tuple): a list or tuple with tools want mandatory info
            subsection (str): the second level on config.yaml (ie 3 level)
        Returns:
            :class:`list`: List of files name with or without extension , with specify extension include on folder
            :class:`list`: List of all extension found
        Raises:
            NotADirectoryError: If config.yaml data `path` does not exist.
        """
        path_value = self.get_config_value(section=section, key=key, subsection=subsection)
        if path_value:
            path = Path(path_value).resolve().as_posix() + "/"
            if (not Path(path).exists() or not Path(path).is_dir()) and key not in ["OUTPUT"]:
                raise NotADirectoryError(
                    f'CONFIG FILE CHECKING FAIL : in the "{section}" section, {f"subsection {subsection}" if subsection else ""}, {key} directory "{path}" {"does not exist" if not Path(path).exists() else "is not a valid directory"}')
            else:
                self.set_config_value(section, key, path, subsection)
        elif len(mandatory) > 0:
            raise NotADirectoryError(
                f'CONFIG FILE CHECKING FAIL : in the "{section}" section, {f"subsection {subsection}" if subsection else ""}, {key} directory "{path_value}" {"does not exist" if not Path(path_value).exists() else "is not a valid directory"} but is mandatory for tool: {" ".join(mandatory)}')

    def __check_file(self, section, key, mandatory=[], subsection=None):
        """Check if path is a file if not empty
        :return absolute path file"""
        path_value = self.get_config_value(section=section, key=key, subsection=subsection)
        path = Path(path_value).resolve().as_posix()
        if path_value != "":
            if not Path(path).exists() or not Path(path).is_file():
                raise FileNotFoundError(
                    f'CONFIG FILE CHECKING FAIL : in the {section} section, {f"subsection {subsection}" if subsection else ""},{key} file "{path}" {"does not exist" if not Path(path).exists() else "is not a valid file"}')
            else:
                self.set_config_value(section, key, path, subsection)
        elif len(mandatory) > 0:
            raise FileNotFoundError(
                f'CONFIG FILE CHECKING FAIL : in the "{section}" section, {f"subsection {subsection}" if subsection else ""},{key} file "{path_value}" {"does not exist" if not Path(path_value).exists() else "is not a valid file"} but is mandatory for tool: {" ".join(mandatory)}')

    def __check_dir_or_string (self, section, key, mandatory=[], subsection=None, check_string=False):
        """ similar to check_dir"""
        path_value = self.get_config_value(section=section, key=key, subsection=subsection)
        if path_value:
            path = Path(path_value).resolve().as_posix() + "/"
            # it is a path
            if path_value != "" and "/" in path_value:
                if (not Path(path).exists() or not Path(path).is_dir()) and key not in ["OUTPUT"]:
                    raise NotADirectoryError(
                        f'CONFIG FILE CHECKING FAIL : in the "{section}" section, {f"subsection {subsection}" if subsection else ""}, {key} directory "{path}" {"does not exist" if not Path(path).exists() else "is not a valid directory"}')
                else:
                    self.set_config_value(section, key, path, subsection)
            # it is not a path
            elif path_value != "" and not "/" in path_value and check_string:
                self.set_config_value(section, key, path_value, subsection)
            # it is empty
            elif path_value == "" and not "/" in path_value and check_string:
                raise ValueError(
                    f'CONFIG FILE CHECKING FAIL : in the {section} section, {f"subsection {subsection}" if subsection else ""},{key} Value "{path_value}" is empty')
        elif len(mandatory) > 0:
            raise NotADirectoryError(
                f'CONFIG FILE CHECKING FAIL : in the "{section}" section, {f"subsection {subsection}" if subsection else ""}, {key} directory "{path_value}" {"does not exist" if not Path(path_value).exists() else "is not a valid directory"} but is mandatory for tool: {" ".join(mandatory)}')

    def __check_file_or_string(self, section, key, mandatory=[], subsection=None, check_string=False):
        """Check if path is a file if not empty
        :return absolute path file"""
        path_value = self.get_config_value(section=section, key=key, subsection=subsection)
        path = Path(path_value).resolve().as_posix()
        # it is a path
        if path_value != "" and "/" in path_value:
            if not Path(path).exists() or not Path(path).is_file():
                raise FileNotFoundError(
                    f'CONFIG FILE CHECKING FAIL : in the {section} section, {f"subsection {subsection}" if subsection else ""},{key} file "{path}" {"does not exist" if not Path(path).exists() else "is not a valid file"}')
            else:
                self.set_config_value(section, key, path, subsection)
        # it is not a path
        elif path_value != "" and not "/" in path_value and check_string:
            self.set_config_value(section, key, path_value, subsection)
        # it is empty
        elif path_value == "" and not "/" in path_value and check_string:
            raise ValueError(
                f'CONFIG FILE CHECKING FAIL : in the {section} section, {f"subsection {subsection}" if subsection else ""},{key} Value "{path_value}" is empty')
        elif len(mandatory) > 0:
            raise FileNotFoundError(
                f'CONFIG FILE CHECKING FAIL : in the "{section}" section, {f"subsection {subsection}" if subsection else ""}:{key} file "{path_value}" {"does not exist" if not Path(path_value).exists() else "is not a valid file"} but is mandatory for tool: {" ".join(mandatory)}')


    def __check_tools_config(self, tool, mandatory=[]):
        """Check if path is a file if not empty
        :return absolute path file"""
        tool_OK = False
        if tool in ["FLAGSTATS"]:
            tool = "SAMTOOLS"
        def check_singularity(tool):
            section = "SINGULARITY"
            path_file = self.tools_config[section][tool]
            if re.findall("shub://SouthGreenPlatform/CulebrONT_pipeline", path_file, flags=re.IGNORECASE):
                raise ValueError(
                    f'CONFIG FILE CHECKING FAIL : shub download is obsolete')
            else :
                path_file = eval(f"f'{path_file}'")
                path = Path(path_file).resolve().as_posix()
                if path and path_file:
                    if not Path(path).exists() or not Path(path).is_file():
                        raise FileNotFoundError(
                            f'CONFIG FILE CHECKING FAIL : please check tools_config.yaml in the {section} section, {tool} file "{path}" {"does not exist" if not Path(path).exists() else "is not a valid file"}')
                    else:
                        self.tools_config[section][tool] = path

        # If only singularity tools
        if not self.use_env_modules and self.use_singularity:
            check_singularity("REPORT")     # check singularity image for Report
            check_singularity("TOOLS")      # check singularity image for conda (contains a lot of conda envs)
            tool_OK = True

        # If only envmodule
        if self.use_env_modules and not self.use_singularity:
            envmodule_key = self.tools_config["ENVMODULE"][tool]
            if not envmodule_key:
                raise ValueError(
                    f'CONFIG FILE CHECKING FAIL : please check tools_config.yaml in the "ENVMODULE" section, {tool} is empty')
            tool_OK = True

        # If envmodule and singularity
        if self.use_env_modules and self.use_singularity:
            raise ValueError(
                f"CONFIG FILE CHECKING FAIL : Use env-module or singularity but don't mix them")

        if len(mandatory) > 0 and not tool_OK:
            raise FileNotFoundError(
                f'CONFIG FILE CHECKING FAIL : please check tools_config.yaml in the  {tool} params, please append Singularity or module load, is mandatory for tool: {" ".join(mandatory)}')

    @property
    def string_to_dag(self):
        """ return command line for rule graph """
        return f"""snakemake -s {self.snakefile} {'--use-singularity' if self.use_singularity else ''} {'--use-envmodules' if self.use_env_modules else ''}  --rulegraph"""        

    def __var_2_bool(self, key, tool, to_convert):
        """convert to boolean"""
        if isinstance(type(to_convert), bool):
            return to_convert
        elif f"{to_convert}".lower() in ("yes", "true", "t"):
            return True
        elif f"{to_convert}".lower() in ("no", "false", "f"):
            return False
        else:
            raise TypeError(
                f'CONFIG FILE CHECKING FAIL : in the "{key}" section, "{tool}" key: "{to_convert}" is not a valide boolean')

    def __build_tools_activated(self, key, allow, mandatory=False):
        tools_activate = []
        for tool, activated in self.config[key].items():
            if tool in allow:
                boolean_activated = self.__var_2_bool(key, tool, activated)
                if boolean_activated:
                    tools_activate.append(tool)
                    self.config[key][tool] = boolean_activated
                    self.__check_tools_config(tool, [tool])
            else:
                raise ValueError(f'CONFIG FILE CHECKING FAIL : {key} {tool} not allow on CulebrONT"')
        if len(tools_activate) == 0 and mandatory:
            raise ValueError(f"CONFIG FILE CHECKING FAIL : you need to set True for at least one {key} from {allow}")
        return tools_activate

    def __build_quality_step_list(self, only_last=False):
        last_steps_list = []
        suffix = ""
        if bool(self.config["FIXSTART"]):
            suffix = "_STARTFIXED"
        if self.correction_tools_activated:
            step = "CORRECTION"
            for corrector in self.correction_tools_activated:
                last_steps_list.append(f"STEP_{step}_{corrector}{suffix if step in self.pipeline_stop else ''}")
            if only_last:
                return last_steps_list

        if self.polishing_tools_activated:
            step = "POLISHING"
            for polisher in self.polishing_tools_activated:
                last_steps_list.append(f"STEP_{step}_{polisher}{suffix if step in self.pipeline_stop else ''}")
            if only_last:
                return last_steps_list
        if self.assembly_tools_activated:
            step = "ASSEMBLY"
            last_steps_list.append(f"STEP_{step}{suffix if step in self.pipeline_stop else ''}")
            # for assembler in self.assembly_tools_activated:
            # last_steps_list.append(f"STEP_{step}_{assembler}{suffix}" )
            if only_last: return last_steps_list
        return last_steps_list

    def __get_last_step(self):
        if self.correction_tools_activated:
            return "CORRECTION"
        if self.polishing_tools_activated:
            return "POLISHING"
        if self.assembly_tools_activated:
            return "ASSEMBLY"

    def __check_config_dic(self):
        """Configuration file checking"""
        # check tools activation
        self.assembly_tools_activated = self.__build_tools_activated("ASSEMBLY", AVAIL_ASSEMBLY, True)
        self.polishing_tools_activated = self.__build_tools_activated("POLISHING", AVAIL_POLISHING)
        self.correction_tools_activated = self.__build_tools_activated("CORRECTION", AVAIL_CORRECTION)
        self.quality_tools_activated = self.__build_tools_activated("QUALITY", AVAIL_QUALITY)
        self.pipeline_stop = self.__get_last_step()

        # check mandatory directory
        self.__check_dir(section="DATA", key="OUTPUT")
        self.__check_dir(section="DATA", key="FASTQ", mandatory=self.assembly_tools_activated)

        # check if fastq file for assembly
        self.fastq_files_list, fastq_files_list_ext = get_files_ext(self.get_config_value('DATA', 'FASTQ'),
                                                                    ALLOW_FASTQ_EXT)
        if not self.fastq_files_list:
            raise ValueError(
                f"CONFIG FILE CHECKING FAIL : you need to append at least on fastq with extension on {ALLOW_FASTQ_EXT}")
        # check if all fastq have the same extension
        if len(fastq_files_list_ext) > 1:
            raise ValueError(
                f"CONFIG FILE CHECKING FAIL : Please use only the same format for assembly FASTQ data, not: {fastq_files_list_ext}")
        else:
            self.fastq_files_ext = fastq_files_list_ext[0]
        # check if fastq are gzip
        if "gz" in self.fastq_files_ext:
            self.fastq_gzip = True

        ##### CHECK KAT
        if bool(self.config["QUALITY"]["KAT"]) or bool(self.config["CORRECTION"]["PILON"]) or bool(
                self.config["QUALITY"]['FLAGSTATS']):
            self.__check_dir(section="DATA", key="ILLUMINA", mandatory=["KAT", "PILON", "FLAGSTATS"])
            self.illumina_files_list, illumina_files_list_ext = get_files_ext(self.get_config_value('DATA', 'ILLUMINA'),
                                                                              ALLOW_FASTQ_EXT)

            if not self.illumina_files_list:
                raise ValueError(
                    f"CONFIG FILE CHECKING FAIL : you need to append at least on fastq illumina with extension on {ALLOW_FASTQ_EXT}")
            # check if all fastq have the same extension
            if len(illumina_files_list_ext) > 1:
                raise ValueError(
                    f"CONFIG FILE CHECKING FAIL : Please use only the same format for ILLUMINA FASTQ data, not: {illumina_files_list_ext}")
            else:
                self.illumina_files_ext = illumina_files_list_ext[0]
            # check if fastq are gzip
            if "gz" in self.illumina_files_ext:
                self.illumina_gzip = True

            self.R1, self.R2 = self.__split_illumina()

        # build quality_step and last_steps_list; function test if FIXSTART to append only to last step
        self.last_steps_list = self.__build_quality_step_list(only_last=True)
        self.quality_step = self.__build_quality_step_list(only_last=False)

        # check files if QUAST
        if bool(self.config["QUALITY"]["QUAST"]):
            self.__check_file(section="DATA", key="REF")
        genome_pb = convert_genome_size(self.get_config_value('DATA', 'GENOME_SIZE'))
        self.set_config_value(section="params", subsection="QUAST", key="GENOME_SIZE_PB", value=genome_pb)

        # check files if MAUVE
        if bool(self.config["MSA"]["MAUVE"]):
            self.__check_file(section="DATA", key="REF", mandatory=["MAUVE"])
            # Make sure running Mauve makes sense
            if len(self.assembly_tools_activated) < 2 and len(self.quality_step) < 2:
                raise ValueError(
                    "CONFIG FILE CHECKING ERROR : MAUVE is irrelevant if you have a single final assembly as your config file implies (need more than one assembler and/or a correction step). Mauve will not be run !! ")

        # check BUSCO database if activate
        if bool(self.config["QUALITY"]["BUSCO"]):
            #self.__check_dir(section='params', subsection='BUSCO', key='DATABASE', mandatory=["BUSCO"])
            self.__check_dir_or_string(section='params', subsection='BUSCO', key='DATABASE', mandatory=["BUSCO"], check_string=True)

        # check DIAMOND database if activate
        if bool(self.config["QUALITY"]["BLOBTOOLS"]):
            self.__check_file(section='params', subsection='DIAMOND', key='DATABASE',
                              mandatory=["BLOBTOOLS", 'DIAMOND'])

        # check if NANOPOLISH activate, if true compare fastq and fast5 files
        if "NANOPOLISH" in self.correction_tools_activated:
            self.__check_dir(section="DATA", key="FAST5", mandatory=["NANOPOLISH"])
            fast5_files_list = get_dir(self.config['DATA']['FAST5'])
            fastq_files_list, _ = get_files_ext(self.config['DATA']['FASTQ'], ALLOW_FASTQ_EXT, add_ext=False)

            if set(fastq_files_list) - set(fast5_files_list):
                raise ValueError(
                    f"CONFIG FILE CHECKING ERROR : You don't have a fast5 repository for each of your fastq file (they should have the same name). This can raise a problem if you choose to use Nanopolish. Please check your data.:\n\t- fast5_files_list:{fast5_files_list}\n\t- fastq_files_list: {fastq_files_list}\n\n")

        # check Medaka config if activate
        if bool(self.config['CORRECTION']['MEDAKA']):
            # check singularity image
            if not bool(self.config['params']['MEDAKA']['MEDAKA_TRAIN_WITH_REF']):
                #self.__check_file(section='params', subsection='MEDAKA', key='MEDAKA_MODEL_PATH', mandatory=["MEDAKA"])
                self.__check_file_or_string(section='params', subsection='MEDAKA', key='MEDAKA_MODEL_PATH',
                                            mandatory=["MEDAKA"], check_string=True)
            else:
                self.__check_file(section='DATA', key='REF', mandatory=["MEDAKA"])

        # Check racon round
        if bool(self.config['POLISHING']['RACON']) and not (
                0 < int(self.config['params']['RACON']['RACON_ROUNDS']) < 10):
            raise ValueError(
                f"CONFIG FILE CHECKING ERROR : You have activated RACON, but RACON_ROUNDS is invalid, 0 < RACON_ROUNDS={self.config['params']['RACON']['RACON_ROUNDS']} < 10 . \n")

        ##############################
        # check workflow compatibility
        if not bool(self.config['POLISHING']['RACON']) and bool(self.config['ASSEMBLY']['MINIASM']):
            logger.warning(
                f"\nWARNING: RACON is automatically launched (2 rounds by default) for minipolish if MINIASM is activated !! . \n")

        # check size of genome
        if int(convert_genome_size(self.get_config_value('DATA', 'GENOME_SIZE'))) >= 50000000 and self.get_config_value('MSA', 'MAUVE'):
            logger.warning(
                f"WARNING: CONFIG FILE CHECKING WARNING : MAUVE if fixed to FALSE because genome size  >= 50 mb !! \n")
            self.set_config_value('MSA', 'MAUVE', False)

        # Make sure running fixstart makes sense
        if not bool(self.config['CIRCULAR']) and bool(self.config['FIXSTART']):
            raise ValueError(
                f"CONFIG FILE CHECKING ERROR : FIXSTART is irrelevant if you have not activated CIRCULAR. FIXSTART will not be run !! \n")

        # if you want run mauve fixstart has to be activated
        if bool(self.config['CIRCULAR']) and not bool(self.config['FIXSTART']):
            raise ValueError(
                f"CONFIG FILE CHECKING ERROR : FIXSTART must be activated if CIRCULAR is TRUE on config file.  !! \n")

        # check for BLOBTOOLS
        if bool(self.config["QUALITY"]["BLOBTOOLS"]):
            self.__check_file(section='params', subsection='BLOBTOOLS', key='NAMES', mandatory=["BLOBTOOLS"])
            self.__check_file(section='params', subsection='BLOBTOOLS', key='NODES', mandatory=["BLOBTOOLS"])

        #############################################
        # Build variables name for files
        if bool(self.config['CIRCULAR']):
            self.add_circular_name = "CIRCULARISED"
            self.TMP = "TMP"
            self.TCM = "FALSE"
        else:
            self.add_circular_name = ""
            self.TMP = ""
            self.TCM = ""

        if not bool(self.config['POLISHING']['RACON']) and bool(self.config['CIRCULAR']):
            self.TAG = "TMPTAG"
        else:
            self.TAG = ""

        # ############################### DEF ####################################
        self.nb_racon_rounds = '2' if not bool(self.config['POLISHING']['RACON']) else str(
            self.config['params']['RACON']['RACON_ROUNDS'])
        self.nb_pilon_rounds = '1' if not bool(self.config['CORRECTION']['PILON']) else str(
            self.config['params']['PILON']['PILON_ROUNDS'])

    def __repr__(self):
        return f"{self.__class__}({pprint.pprint(self.__dict__)})"



#!/usr/bin/env python3
import os
import click
import culebrONT
from culebrONT.global_variable import *
from culebrONT.usefull_function import get_install_mode, check_privileges


@click.group(help=click.secho(culebrONT.description_tools, fg='green', nl=False), context_settings={'help_option_names': ('-h', '--help'),"max_content_width":800},
             invoke_without_command=True, no_args_is_help=True)
@click.option('--restore', '-r', is_flag=True, required=False, default=False, show_default=True, help='Restore installation mode (need root or sudo)')
@click.version_option(culebrONT.__version__, '--version', '-v')
@click.pass_context
def main(ctx, restore):
    if ctx.invoked_subcommand is None and restore and check_privileges():
        if CULEBRONT_MODE.exists():
            CULEBRONT_MODE.unlink(missing_ok=False)
            click.secho(f"\n    Remove installation mode, now run:\n    culebrONT install_local or install_cluster\n\n", fg="yellow")
        else:
            click.secho(f"\n    No reset need, culebrONT not install !!!!!\n    Please run: culebrONT install_local or install_cluster !!!!\n\n", fg="red")
    pass


# Hack for build docs with unspecified install
args = str(sys.argv)
if "sphinx" in args:
    main.add_command(culebrONT.run_cluster)
    main.add_command(culebrONT.create_cluster_config)
    main.add_command(culebrONT.create_config)
    main.add_command(culebrONT.edit_tools)
    main.add_command(culebrONT.run_local)
    main.add_command(culebrONT.install_cluster)
    main.add_command(culebrONT.install_local)
    main.add_command(culebrONT.test_install)
else:
    mode = get_install_mode()
    if mode == "cluster":
        main.add_command(culebrONT.test_install)
        main.add_command(culebrONT.run_cluster)
        main.add_command(culebrONT.create_cluster_config)
        main.add_command(culebrONT.create_config)
        main.add_command(culebrONT.edit_tools)
    elif mode == "local":
        main.add_command(culebrONT.test_install)
        main.add_command(culebrONT.run_local)
        main.add_command(culebrONT.create_config)
    else:
        main.add_command(culebrONT.install_cluster)
        main.add_command(culebrONT.install_local)





if __name__ == '__main__':
    main()



#!/usr/bin/env python3
"""
Takes several final polished and fixstarted assembly fasta files,
create corresponding symbolic link in output directory and
run progressive Mauve to create a multiple alignment of these genomes
"""

import os
import subprocess

fasta_files = snakemake.input.liste
out_dir = snakemake.params.out_dir
dir = snakemake.params.dir
xmfa = snakemake.output.xmfa
dir_aggregated = snakemake.params.dir_aggregated
log_e = snakemake.log.error
log_o = snakemake.log.output

dest_paths = []
for src in fasta_files:
    dst = os.path.join(out_dir, os.path.basename(src))
    if os.path.exists(dst):
        os.remove(dst)
    os.symlink(src, dst)
    dest_paths.append(dst)
stringList = ' '.join([str(item) for item in dest_paths])

#recovery version of mauve
version_file = f"{dir}/versions/MAUVE-version.txt"
cmd = f"progressiveMauve --version "
subprocess.run(cmd, shell=True, capture_output=False, stderr=open(version_file, 'w'))

# convert list on string
cmd = f"progressiveMauve --output={xmfa} {stringList}"

#launching MSA
subprocess.run(cmd, shell=True, capture_output=False, stdout=open(log_o, 'w'), stderr=open(log_e, 'w'))

#rename fasta on xmfa
subst = f's\'|{dir_aggregated}||\'ig'
cmd = f"sed -i {subst} {xmfa}"
subprocess.run(cmd, shell=True, capture_output=False)



import click
from click import Abort
from pathlib import Path
from shutil import rmtree, copyfile, unpack_archive
import culebrONT
import os
import re
from cookiecutter.main import cookiecutter
from culebrONT.global_variable import *
from culebrONT.usefull_function import command_required_option_from_option, multiprocessing_download, get_install_mode

required_options = {
    True: 'modules_dir',
    False: 'scheduler'
}

def create_bash_completion():
    bashrc_file = Path("~/.bashrc").expanduser().as_posix()
    import subprocess
    output = subprocess.run(
        ["bash", "-c", "echo ${BASH_VERSION}"], stdout=subprocess.PIPE
    )
    match = re.search(r"^(\d+)\.(\d+)\.\d+", output.stdout.decode())
    if match is not None:
        major, minor = match.groups()
    if major < "4" or (major == "4" and minor < "4"):
        click.secho(f"\n    WARNNING Shell completion is not supported for Bash versions older than 4.4.", fg="red", nl=False)
        # raise RuntimeError("Shell completion is not supported for Bash versions older than 4.4.")
    else:
        if not Path(f"{CULEBRONT_PATH}/culebrONT-complete.sh").exists():
            build_completion = f"_CULEBRONT_COMPLETE=bash_source culebrONT > {CULEBRONT_PATH}/culebrONT-complete.sh"
            os.system(build_completion)
        with open(bashrc_file, "r") as bash_file_read:
            if not [True for line in bash_file_read if "CULEBRONT" in line]:
                with open(bashrc_file, "a") as bash_file_open:
                    append_bashrc = f"\n#Add autocompletion for CULEBRONT\n. {CULEBRONT_PATH}/culebrONT-complete.sh"
                    bash_file_open.write(append_bashrc)
                    click.secho(f"\n    INSTALL autocompletion for CULEBRONT on {bashrc_file} with command {append_bashrc}\nUpdate with commande:\nsource ~/.bashrc",
                                fg="yellow")
            else:
                path_culebront = ""
                with open(bashrc_file, "r") as bash_file_read:
                    for line in bash_file_read:
                        if "CULEBRONT" in line:
                            path_bash = bash_file_read.readline().strip()
                            path_culebront = f"{line}{path_bash}"
                load = f"{path_bash[2:]}"
                if f"{CULEBRONT_PATH}/culebrONT-complete.sh" != load:
                    click.secho(
                        f"\n    WARNNING autocompletion for CULEBRONT  already found on {bashrc_file}, with other path please fix the good:",
                        fg="red", nl=False)
                    click.secho( f"\n    Load on bashrc: {load}\n    New install:    {CULEBRONT_PATH}/culebrONT-complete.sh", fg='bright_red')


def create_envmodules(modules_dir):
    from culebrONT import MODULE_FILE
    modules_dir = Path(modules_dir)
    modules_dir.mkdir(parents=True, exist_ok=True)
    modules_dir.joinpath(f"{culebrONT.__version__}").open("w").write(MODULE_FILE)
    click.edit(require_save=False, extension='', filename=modules_dir.joinpath(f"{culebrONT.__version__}").as_posix())
    click.secho(f"\n    Success install module file for version {culebrONT.__version__} on path '{modules_dir}'", fg="yellow")

def clean_home():
    if Path("~/.config/CulebrONT/").expanduser().exists():
        rmtree(Path("~/.config/CulebrONT/").expanduser().as_posix())

def check_and_download_singularity():
    # check if already download if true pop from list
    SINGULARITY_URL_FILES_DOWNLOAD = []
    for imgs_list in SINGULARITY_URL_FILES:
        url, path_install = imgs_list
        if not Path(path_install).exists():
            SINGULARITY_URL_FILES_DOWNLOAD.append(imgs_list)
        else:
            click.secho(f"    File: {path_install} already download, done.", fg="yellow", nl=True)
    results = multiprocessing_download(SINGULARITY_URL_FILES_DOWNLOAD)
    for r in results:
        click.secho(r, fg="blue")
    click.secho(f"\n    WARNNING please check if binding is active on your singularity configuration, see https://sylabs.io/guides/3.0/user-guide/bind_paths_and_mounts.html", fg = "bright_red")


@click.command("install_cluster", short_help='Install CulebrONT on HPC cluster', context_settings=dict(max_content_width=800),
               cls=command_required_option_from_option('create_envmodule', required_options), no_args_is_help=True)
@click.option('--scheduler', '-s', default="slurm", type=click.Choice(['slurm', 'sge', 'lsf'], case_sensitive=False),
              prompt='Choose your HPC scheduler', show_default=True, help='Type the HPC scheduler')
@click.option('--env', '-e', default="modules", type=click.Choice(['modules', 'singularity'], case_sensitive=False),
              prompt='Choose mode for tools dependencies', show_default=True, help='Mode for tools dependencies ')
@click.option('--bash_completion/--no-bash_completion', is_flag=True, required=False, default=True, show_default=True,
              help='Allow bash completion of culebrONT commands on the bashrc file')
@click.option('--create_envmodule/--no-create_enmodules', is_flag=True, required=False, default=False, show_default=True,
              help='Create a env module file allowing load culebrONT in a cluster')
@click.option('--modules_dir', '-m', default=None,
              type=click.Path(exists=False, dir_okay=True, file_okay=False, readable=True, resolve_path=True),
              required=False, show_default=True, help='Directory used to save the module created by --create_envmodule parameter', is_eager=True)
def install_cluster(scheduler, env, bash_completion, create_envmodule, modules_dir):
    """Run installation for HPC cluster"""
    # add file for installation mode:
    mode_file = CULEBRONT_MODE
    # rm previous install (ie @julie cluster then local)
    clean_home()
    # build default profile path
    default_profile = CULEBRONT_PROFILE
    def fail():
        rmtree(default_profile, ignore_errors=True)
        mode_file.unlink(missing_ok=True)
        click.secho(f"\n    INSTALL FAIL remove  already install files: {default_profile} ", fg="red")
        exit()

    # test if install already run
    try:
        if default_profile.exists() and click.confirm(click.style(f'    Profile "{default_profile}" exist do you want to remove and continue?\n\n', fg="red"), default=False, abort=True):
            rmtree(default_profile, ignore_errors=True)
        default_profile.mkdir(exist_ok=True)

        default_cluster = CULEBRONT_PATH.joinpath("install_files",f"cluster_config_{scheduler.upper()}.yaml")
        copyfile(default_cluster, default_profile.joinpath("cluster_config.yaml"))

        # Download cookiecutter of scheduler
        click.secho(f"    You choose '{scheduler}' as scheduler. Download cookiecutter:\n    https://github.com/Snakemake-Profiles/{scheduler.lower()}.git", fg="yellow")
        cookiecutter(f'https://github.com/Snakemake-Profiles/{scheduler.lower()}.git',
                     checkout=None,
                     no_input=True,
                     extra_context={"profile_name": f'',
                                    "sbatch_defaults": "--export=ALL",
                                    "cluster_config": f"{default_profile}/cluster_config.yaml",
                                    "advanced_argument_conversion": 1,
                                    "cluster_name": ""
                                    },
                     replay=False, overwrite_if_exists=True,
                     output_dir=f'{default_profile}', config_file=None,
                     default_config=False, password=None, directory=None, skip_if_file_exists=True)

        try:
            # default slurm cookiecutter not contain all snakemake variables
            if scheduler == "slurm":
                extra_slurm = f"use-envmodules: {'true' if env == 'modules' else 'false' }\nuse-singularity: {'true' if env == 'singularity' else 'false' }\nrerun-incomplete: true\nprintshellcmds: true"
                with open(f"{default_profile}/config.yaml", "a") as config_file:
                    config_file.write(extra_slurm)

            # Edit cluster_config.yaml, config.yaml and tools_path.yaml
            if click.confirm(click.style(f'\n    Now Edit file cluster_config.yaml according to your HPC:\n    More info on on {DOCS} \n    (enter to continue)', fg="blue"), default=True, abort=True, show_default=True):
                click.edit(require_save=False, extension='.yaml', filename=f"{default_profile}/cluster_config.yaml")

            if click.confirm(click.style(f'\n    Now Edit file config.yaml according to your HPC:\n    More info on {DOCS}\n    (enter to continue)', fg="blue"), default=True, abort=True, show_default=True):
                click.edit(require_save=False, extension='.yaml', filename=f"{default_profile}/config.yaml")

            if env != "singularity":
                if click.confirm(click.style(f'\n    Now Edit file tools_path.yaml according to your HPC:\n    More info on {DOCS}\n    (enter to continue)', fg="blue"), default=True, abort=True, show_default=True):
                    click.edit(require_save=False, extension='.yaml', filename=f"{CULEBRONT_TOOLS_PATH}")
            click.secho(f"\n    Profile is success install on {default_profile}", fg="yellow")
        except Abort:
            fail()
    except Abort:
        click.secho(f"\n    Profile is already created, skipping {default_profile}", fg="yellow")
    except Exception as e:
        print(e)
        fail()
    try:
        # if singularity activation
        if env == 'singularity':
            # check if already download
            check_and_download_singularity()
        # if envmodule activation
        if create_envmodule:
            create_envmodules(modules_dir)
        # export to add bash completion
        if bash_completion:
            create_bash_completion()
        click.secho(f"\n    Congratulations, you have successfully installed culebrONT !!!\n\n", fg="green", bold=True)
        mode_file.open("w").write("cluster")
    except Exception as e:
        click.secho(f"\n    ERROR : an error was detected, please check {e}",  fg="red")
        fail()


@click.command("install_local", short_help='Install CulebrONT on local computer', context_settings=dict(max_content_width=800), no_args_is_help=False)
@click.option('--bash_completion/--no-bash_completion', is_flag=True, required=False, default=True, show_default=True,
              help='Allow bash completion of culebrONT commands on the bashrc file')
def install_local(bash_completion):
    """
    \b
    Run installation for local computer with download the singularity images.
    """
    # add file for installation mode:
    mode_file = CULEBRONT_MODE
    # rm previous install (ie @julie cluster then local)
    clean_home()
    # add path to download
    CULEBRONT_PATH.joinpath("containers").mkdir(exist_ok=True, parents=True)
    try:

        check_and_download_singularity()
        # export to add bash completion
        if bash_completion:
           create_bash_completion()
        click.secho(f"\n    Congratulations, you have successfully installed culebrONT !!!\n\n", fg="green", bold=True)
        mode_file.open("w").write("local")
    except Exception as e:
        mode_file.unlink(missing_ok=True)
        click.secho(f"\n    ERROR : an error was detected, please check {e}", fg="red")
        exit()


@click.command("test_install", short_help='Test culebrONT with data_test after install (local or cluster)', context_settings=dict(max_content_width=800), no_args_is_help=False)
@click.option('--data_dir', '-d', default=None,
              type=click.Path(exists=False, file_okay=False, dir_okay=True, readable=True, resolve_path=True),
              required=True, show_default=False, help='Path to download datatest and create config.yaml to run test')
def test_install(data_dir):
    # create dir test and configure config.yaml
    data_dir = Path(data_dir)
    data_config_path = data_dir.joinpath("data_test_config.yaml")
    data_dir.mkdir(parents=True, exist_ok=True)
    txt =  CULEBRONT_CONFIG_PATH.open("r").read().replace("./Data-Xoo-sub",f"{data_dir}/Data-Xoo-sub").replace("./CulebrONT_OUTPUT/",f"{data_dir}/CulebrONT_OUTPUT/")
    data_config_path.open("w").write(txt)

    # download data
    download_zip = data_dir.joinpath("Data-Xoo-sub.zip")
    results = multiprocessing_download([("https://itrop.ird.fr/culebront_utilities/Data-Xoo-sub.zip", download_zip.as_posix())], threads=1)
    for r in results:
        click.secho(r, fg="blue")
    unpack_archive(download_zip, data_dir)
    download_zip.unlink()

    # build commande line
    mode = get_install_mode()
    cmd = f"\n    culebrONT {'run_cluster' if mode == 'cluster' else 'run_local --threads 1'} --config {data_config_path}\n\n"
    click.secho(cmd, fg='bright_blue')


import numpy as np
from itertools import groupby
import json
import sys
import pandas as pd
from pathlib import Path
from collections import OrderedDict

class AutoVivification(OrderedDict):
    """
    Implementation of perl's autovivification feature.

    Example:

    >>> a = AutoVivification()
    >>> a[1][2][3] = 4
    >>> a[1][3][3] = 5
    >>> a[1][2]['test'] = 6
    >>> print a
    >>> {1: {2: {'test': 6, 3: 4}, 3: {3: 5}}}

    """

    def __getitem__(self, item):
        try:
            return dict.__getitem__(self, item)
        except KeyError:
            value = self[item] = type(self)()
            return value

def replace_all(repls, str):
    """
    Function that take a dictionnary and text variable and return text variable with replace 'Key' from dictionnary with 'Value'.

    :param repls: a python dictionary
    :type repls: dict()
    :param str: a string where remplace some words
    :type str: str()
    :rtype: str()
    :return: - txt with replace 'Key' of dictionnary with 'Value' in the input txt

    Example:
        >>> text =  "i like apples, but pears scare me"
        >>> print(replace_all({"apple": "pear", "pear": "apple"}, text))
        i like pears, but apples scare me
    """
    import re
    return re.sub('|'.join(re.escape(key) for key in repls.keys()), lambda k: repls[k.group(0)], str)



def fasta_iter(fasta_file):
    """Takes a FASTA file, and produces a generator of Header and Sequences.
    This is a memory-efficient way of analyzing a FASTA files -- without
    reading the entire file into memory.

    Parameters
    ----------
    fasta_file : str
        The file location of the FASTA file

    Returns
    -------
    header: str
        The string contained in the header portion of the sequence record
        (everything after the '>')
    seq: str
        The sequence portion of the sequence record
    """

    fh = open(fasta_file)
    fa_iter = (x[1] for x in groupby(fh, lambda line: line[0] == ">"))
    for header in fa_iter:
        # drop the ">"
        header = next(header)[1:].strip()
        # join all sequence lines to one.
        seq = "".join(s.upper().strip() for s in next(fa_iter))
        yield header, seq


def read_genome(fasta_file):
    """Takes a FASTA file, and produces 2 lists of sequence lengths. It also
    calculates the GC Content, since this is the only statistic that is not
    calculated based on sequence lengths.

    Parameters
    ----------
    fasta_file : str
        The file location of the FASTA file

    Returns
    -------
    contig_lens: list
        A list of lengths of all contigs in the genome.
    scaffold_lens: list
        A list of lengths of all scaffolds in the genome.
    gc_cont: float
        The percentage of total basepairs in the genome that are either G or C.
    """

    gc = 0
    total_len = 0
    contig_lens = []
    scaffold_lens = []
    for _, seq in fasta_iter(fasta_file):
        scaffold_lens.append(len(seq))
        if "NN" in seq:
            contig_list = seq.split("NN")
        else:
            contig_list = [seq]
        for contig in contig_list:
            if len(contig):
                gc += contig.count('G') + contig.count('C')
                total_len += len(contig)
                contig_lens.append(len(contig))
    gc_cont = (gc / total_len) * 100
    return contig_lens, scaffold_lens, gc_cont


def calculate_stats(seq_lens, gc_cont):
    stats = {}
    seq_array = np.array(seq_lens)
    stats['sequence_count'] = seq_array.size
    stats['gc_content'] = gc_cont
    sorted_lens = seq_array[np.argsort(-seq_array)]
    stats['longest'] = int(sorted_lens[0])
    stats['shortest'] = int(sorted_lens[-1])
    stats['median'] = np.median(sorted_lens)
    stats['mean'] = np.mean(sorted_lens)
    stats['total_bps'] = int(np.sum(sorted_lens))
    csum = np.cumsum(sorted_lens)
    for level in [10, 20, 30, 40, 50]:
        nx = int(stats['total_bps'] * (level / 100))
        csumn = min(csum[csum >= nx])
        l_level = int(np.where(csum == csumn)[0])
        n_level = int(sorted_lens[l_level])

        stats['L' + str(level)] = l_level
        stats['N' + str(level)] = n_level
    return stats


if __name__ == "__main__":
    files_list =  snakemake.input.fasta_list
    outfile =  snakemake.output.csv
    sample = snakemake.params.sample
    stat_output = AutoVivification()

    for assembly_file in sorted(files_list):
        assembly_file_path = Path(assembly_file)
        assembler, step, _ = assembly_file_path.stem.split("-")
        step = replace_all({"STEP_": "", "_STARTFIXED": ""}, step)
        contig_lens, scaffold_lens, gc_cont = read_genome(assembly_file)
        scaffold_stats = calculate_stats(scaffold_lens, gc_cont)
        stat_output[assembler][step] = scaffold_stats
    df=pd.DataFrame.from_dict(stat_output)
    dataframe_stats= df.T.stack().apply(pd.Series)
    with open(outfile, "w") as csv_out:
        dataframe_stats.to_csv(csv_out)




import pandas as pd
import argparse
from pathlib import Path
from collections import OrderedDict
from pprint import pprint as pp
import re
version = "0.0.1"

def replace_all(repls, str):
    """
    Function that take a dictionnary and text variable and return text variable with replace 'Key' from dictionnary with 'Value'.

    :param repls: a python dictionary
    :type repls: dict()
    :param str: a string where remplace some words
    :type str: str()
    :rtype: str()
    :return: - txt with replace 'Key' of dictionnary with 'Value' in the input txt

    Example:
        >>> text =  "i like apples, but pears scare me"
        >>> print(replace_all({"apple": "pear", "pear": "apple"}, text))
        i like pears, but apples scare me
    """
    import re
    return re.sub('|'.join(re.escape(key) for key in repls.keys()), lambda k: repls[k.group(0)], str)


class AutoVivification(OrderedDict):
    """
    Implementation of perl's autovivification feature.

    Example:

    >>> a = AutoVivification()
    >>> a[1][2][3] = 4
    >>> a[1][3][3] = 5
    >>> a[1][2]['test'] = 6
    >>> print a
    >>> {1: {2: {'test': 6, 3: 4}, 3: {3: 5}}}

    """

    def __getitem__(self, item):
        try:
            return dict.__getitem__(self, item)
        except KeyError:
            value = self[item] = type(self)()
            return value

def main():
    
    assembly_list = snakemake.params.assembly_list
    quality_list = snakemake.params.quality_list
    out_repository = snakemake.params.out_dir
    out_stats = snakemake.output.stat
    summary_busco = snakemake.input.summary
    
    dico_busco_time = AutoVivification()

    # f"{output_dir}{{fastq}}/ASSEMBLERS/{{assemblers}}/QUALITY/{{quality_step}}/BUSCO/{{assemblers}}_{{quality_step}}_short-summary-BUSCO.txt",

    for busco_file in sorted(summary_busco):
        busco_file_path = Path(busco_file)
        assembler, step, _ = busco_file_path.stem.split("-")
        
        step = replace_all({"STEP_":"","_STARTFIXED":""}, step)
        with open(busco_file, 'r') as busco:
                for line in busco:
                    line = line.strip()
                    if not line.startswith('#') or not line.startswith('\n'):
                        # print(line)
                        if line.startswith('C'):
                            complete, single, duplicate, fragmented, missing, total = re.search(r"C:(\d+\.+\d+)%\[S:(\d+\.+\d+)%,D:(\d+\.+\d+)%\],F:(\d+\.+\d+)%,M:(\d+\.+\d+)%,n:(\d+)",line).groups()

        dico_busco_time[assembler][step]['Complete'] = f'{complete}%'
        dico_busco_time[assembler][step]['Single'] = f'{single}%'
        dico_busco_time[assembler][step]['Duplicate'] = f'{duplicate}%'
        dico_busco_time[assembler][step]['Fragmented'] = f'{fragmented}%'
        dico_busco_time[assembler][step]['Missing'] = f'{missing}%'
        dico_busco_time[assembler][step]['Total Genes'] = f'{total}'
     

    df=pd.DataFrame.from_dict(dico_busco_time)
    dataframe_busco= df.T.stack().apply(pd.Series)
    with open(out_stats, "w") as busco_file:
        #print(f"dataframe_busco:\n{dataframe_busco}\n")
        dataframe_busco.to_csv(busco_file, index=True)


if __name__ == '__main__':
    main()



import sys
from pathlib import Path

DOCS = "https://culebront-pipeline.readthedocs.io/en/latest/"
GIT_URL = "https://github.com/SouthGreenPlatform/CulebrONT_pipeline"

# Hack for build docs with unspecified path install
args = str(sys.argv)
if "sphinx" in args:
    CULEBRONT_PATH = Path("/Path/to/culebrONT_install/")
else:
    CULEBRONT_PATH = Path(__file__).resolve().parent
CULEBRONT_SNAKEFILE = CULEBRONT_PATH.joinpath("snakefiles", "Snakefile")
CULEBRONT_MODE = CULEBRONT_PATH.joinpath(".mode.txt")
CULEBRONT_SCRIPTS = CULEBRONT_PATH.joinpath("snakemake_scripts")
CULEBRONT_PROFILE = CULEBRONT_PATH.joinpath("default_profile")
CULEBRONT_CONFIG_PATH = CULEBRONT_PATH.joinpath("install_files", "config.yaml")

CULEBRONT_TOOLS_PATH = CULEBRONT_PATH.joinpath("install_files", "tools_path.yaml")
CULEBRONT_USER_TOOLS_PATH = Path("~/.config/CulebrONT/tools_path.yaml").expanduser()
CULEBRONT_ARGS_TOOLS_PATH = Path("~/.config/CulebrONT/tools_path_args.yaml").expanduser()

CULEBRONT_CLUSTER_CONFIG = CULEBRONT_PROFILE.joinpath("cluster_config.yaml")
CULEBRONT_USER_CLUSTER_CONFIG = Path("~/.config/CulebrONT/cluster_config.yaml").expanduser()
CULEBRONT_ARGS_CLUSTER_CONFIG = Path("~/.config/CulebrONT/cluster_config_args.yaml").expanduser()


AVAIL_ASSEMBLY = ("CANU", "FLYE", "MINIASM", "RAVEN", "SMARTDENOVO", "SHASTA")
AVAIL_CORRECTION = ("NANOPOLISH", "MEDAKA", "PILON")
AVAIL_POLISHING = ("RACON")
AVAIL_QUALITY = ("BUSCO", "QUAST", "BLOBTOOLS", "ASSEMBLYTICS", "KAT", "FLAGSTATS")

ALLOW_FASTQ_EXT = (".fastq", ".fq", ".fq.gz", ".fastq.gz")

SINGULARITY_URL_FILES = [('https://itrop.ird.fr/culebront_utilities/singularity_build/Singularity.culebront_tools.sif',
              f'{CULEBRONT_PATH}/containers/Singularity.culebront_tools.sif'),
             ('https://itrop.ird.fr/culebront_utilities/singularity_build/Singularity.report.sif',
              f'{CULEBRONT_PATH}/containers/Singularity.report.sif')]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from pathlib import Path
from .run import run_cluster, run_local
from .edit_files import edit_tools, create_config, create_cluster_config
from .install import install_cluster, install_local, test_install
from .usefull_function import get_version, get_last_version
from .global_variable import *

logo = CULEBRONT_PATH.joinpath('culebront_logo.png').as_posix()


__version__ = get_version()

__doc__ = """Today, assembly a genome using long reads from Oxford Nanopore Technologies is really interesting in 
particular to solve repeats and structural variants in prokaryotic as well as in eukaryotic genomes. Assemblies are 
increasing contiguity and accuracy. The daily increase of data sequences obtained and the fact that more and more 
tools are being released or updated every week, many species are having their genomes assembled and that’s is great … 
“But which assembly tool could give the best results for your favorite organism?” CulebrONT can help you! CulebrONT 
is an open-source, scalable, modular and traceable Snakemake pipeline, able to launch multiple assembly tools in 
parallel, giving you the possibility of circularise, polish, and correct assemblies, checking quality. CulebrONT can 
help to choose the best assembly between all possibilities. """

description_tools = f"""
    Welcome to CulebrONT version: {__version__} ! Created on November 2019
    @author: Julie Orjuela (IRD), Aurore Comte (IRD), Sebastien Ravel (CIRAD), Florian Charriat (INRAE),
    Bao Tram Vi (IRD), François Sabot (IRD) and Sebastien Cunnac (IRD)
    @email: julie.orjuela@ird.fr, aurore.comte@ird.fr

    #     .-+.
    #   `odNNh
    #   +NNd:
    #  .Nh.   ---:`
    #  -Nm`  ````./
    #   oNm/ ```-o-
    #    .yNmo/+/.     .oooooo.               oooo             .o8                  .oooooo.   ooooo      ooo ooooooooooooo
    #    `-+ydNmo.    d8P'  `Y8b              `888            "888                 d8P'  `Y8b  `888b.     `8' 8'   888   `8
    #  `/s+../oNNN:  888          oooo  oooo   888   .ooooo.   888oooo.  oooo d8b 888      888  8 `88b.    8       888
    #  +s/     `hNm  888          `888  `888   888  d88' `88b  d88' `88b `888""8P 888      888  8   `88b.  8       888
    #  os:  ////hNN` 888           888   888   888  888ooo888  888   888  888     888      888  8     `88b.8       888
    #  -so- .//sNNs  `88b    ooo   888   888   888  888    .o  888   888  888     `88b    d88'  8       `888       888
    #   `/oo:.yNd/    `Y8bood8P'   `V88V"V8P' o888o `Y8bod8P'  `Y8bod8P' d888b     `Y8bood8P'  o8o        `8      o888o
    #     -yooo+.
    #   `yNs.`-/oo:
    #   dNo` ....+s+
    #   :shmdddhhy+:

    Please cite our github: {GIT_URL}
    Licencied under CeCill-C (http://www.cecill.info/licences/Licence_CeCILL-C_V1-en.html)
    and GPLv3 Intellectual property belongs to IRD, CIRAD and authors.
    Documentation avail at: {DOCS}
    {get_last_version(url=GIT_URL, current_version=__version__)}"""


MODULE_FILE = f"""#%Module1.0
##

## Required internal variables
set     prefix       {CULEBRONT_PATH.as_posix().strip()}
set     version      {__version__.strip()}

# check if install directory exist
if {{![file exists $prefix]}} {{
    puts stderr "\t[module-info name] Load Error: $prefix does not exist"
    break
    exit 1
}}

## List conflicting modules here
conflict culebrONT

## List prerequisite modules here
module load graphviz/2.40.1

set		fullname	CulebrONT-{__version__.strip()}
set		externalurl	"\n\t{DOCS.strip()}\n"
set		description	"\n\t{__doc__.strip()}

## Required for "module help ..."
proc ModulesHelp {{}} {{
  global description externalurl
  puts stderr "Description - $description"
  puts stderr "More Docs   - $externalurl"
}}

## Required for "module display ..." and SWDB
module-whatis   "loads the [module-info name] environment"

## Software-specific settings exported to user environment

prepend-path PATH $prefix

"""



from pathlib import Path
import urllib.request
from tqdm import tqdm
import os
import click


def check_privileges():
    if not os.environ.get("SUDO_UID") and os.geteuid() != 0:
        click.secho(f"\n    ERROR : You need to run -r, --restore with sudo privileges or as root\n", fg="red")
    else:
        return True


class DownloadProgressBar(tqdm):
    def update_to(self, b=1, bsize=1, tsize=None):
        if tsize is not None:
            self.total = tsize
        self.update(b * bsize - self.n)


def download_url(tuple_value):
    url, output_path = tuple_value
    with DownloadProgressBar(unit='B', unit_scale=True,
                             miniters=1, desc=url.split('/')[-1]) as t:
        urllib.request.urlretrieve(url, filename=output_path, reporthook=t.update_to)


def multiprocessing_download(urls_list, threads = 2):
    from multiprocessing.pool import ThreadPool
    return ThreadPool(threads).imap_unordered(download_url, urls_list)


INSTALL_PATH = Path(__file__).resolve().parent


def get_install_mode():
    """detect install mode"""
    if INSTALL_PATH.joinpath(".mode.txt").exists():
        return INSTALL_PATH.joinpath(".mode.txt").open("r").readline().strip()
    else:
        return "notInstall"


def get_version():
    """Read VERSION file to know current version
    Returns:
        version: actual version read on the VERSION file
    Examples:
        version = get_version()
        print(version)
            1.3.0
    """
    with open(INSTALL_PATH.joinpath("VERSION"), 'r') as version_file:
        return version_file.readline().strip()


def get_last_version(url, current_version):
    """Function for know the last version of Git repo in website"""
    try:
        from urllib.request import urlopen
        from re import search
        import click
        module_mane = url.split('/')[-1]
        HTML = urlopen(f"{url}/tags").read().decode('utf-8')
        str_search = f"{url.replace('https://github.com', '')}/releases/tag/.*"
        lastRelease = search(str_search, HTML).group(0).split("/")[-1].split('"')[0]
        epilogTools = ""
        if str(current_version) != lastRelease:
            if lastRelease < str(current_version):
                epilogTools = click.style(f"\n    ** NOTE: This {module_mane} version ({current_version}) is higher than the production version ({lastRelease}), you are using a dev version\n\n", fg="yellow", bold=True)
            elif lastRelease > str(current_version):
                epilogTools = click.style(f"\n    ** NOTE: The Latest version of {module_mane} {lastRelease} is available at {url}/releases\n\n",fg="yellow", underline=True)
        return epilogTools
    except Exception as e:
        epilogTools = click.style(f"\n    ** ENABLE TO GET LAST VERSION, check internet connection\n{e}\n\n", fg="red")
        return epilogTools


def command_required_option_from_option(require_name, require_map):
    import click
    class CommandOptionRequiredClass(click.Command):
        def invoke(self, ctx):
            require = ctx.params[require_name]
            if require not in require_map:
                raise click.ClickException(click.style(f"Unexpected value for --'{require_name}': {require}\n", fg="red"))
            if ctx.params[require_map[require].lower()] is None:
                raise click.ClickException(click.style(f"With {require_name}={require} must specify option --{require_map[require]} path/to/modules\n", fg="red"))
            super(CommandOptionRequiredClass, self).invoke(ctx)
    return CommandOptionRequiredClass




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.

import sys
import os
import sphinx_rtd_theme
import culebrONT
from culebrONT.global_variable import *

# The short X.Y version.
version = culebrONT.__version__
# The full version, including alpha/beta/rc tags
release = culebrONT.__version__

rst_prolog = f"""
.. |tools_path| replace:: {CULEBRONT_TOOLS_PATH}
"""

# -- Project information -----------------------------------------------------
# General information about the project.
project = 'CulebrONT'
copyright = '2019-2021, J Orjuela (IRD), A Comte (IRD), S Ravel (CIRAD), F Charriat (INRAE), T Vi (IRD, AGI), F Sabot (IRD) and S Cunnac (IRD)'
github_doc_root = 'https://github.com/SouthGreenPlatform/CulebrONT_pipeline/tree/master/docs/'
issues_github_path = 'https://github.com/SouthGreenPlatform/CulebrONT_pipeline/issues'

latex_authors = '''
Julie Orjuela (IRD),\\\\
Aurore Comte (IRD),\\\\
Sebastien Ravel (CIRAD),\\\\
Florian Charriat (INRAE)\\\\
Tram Vi(IRD, AGI),\\\\
Francois Sabot(IRD),\\\\
Sébastien Cunnac(IRD)
'''

# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.

extensions = [
              'sphinx.ext.viewcode',
              'sphinx.ext.autosectionlabel',
              'sphinx_copybutton',
              'sphinx_rtd_theme',
              'sphinx_click'
              ]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix of source filenames.
source_suffix = ['.rst', "md"]

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
exclude_patterns = ['_build']

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

master_doc = 'index'

# -- Options for HTML output ----------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
html_theme = 'sphinx_rtd_theme'

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
html_theme_options = {
    'analytics_id': 'UA-172723859-1',  #  Provided by Google in your dashboard
    'analytics_anonymize_ip': False,
    'logo_only': False,
    'display_version': True,
    'prev_next_buttons_location': 'bottom',
    'style_external_links': False,
    'vcs_pageview_mode': '',
    # 'style_nav_header_background': 'cyan',
    # Toc options
    'collapse_navigation': False,
    'sticky_navigation': True,
    'navigation_depth': 3,
    'includehidden': False,
    'titles_only': False
}

# The name for this set of Sphinx documents.  If None, it defaults to
# "<project> v<release> documentation".
html_title = "CulebrONT"

# A shorter title for the navigation bar.  Default is the same as html_title.
html_short_title = "CulebrONT"

# The name of an image file (relative to this directory) to place at the top
# of the sidebar.
html_logo = '_images/culebront_logo.png'

# The name of an image file (within the static path) to use as favicon of the
# docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
# pixels large.
html_favicon = '_images/culebront_logo2.png'

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']

# If false, no index is generated.
html_use_index = True

# If true, the index is split into individual pages for each letter.
html_split_index = False

# If true, links to the reST sources are added to the pages.
# html_show_sourcelink = True

# If true, "Created using Sphinx" is shown in the HTML footer. Default is True.
html_show_sphinx = False

# If true, "(C) Copyright ..." is shown in the HTML footer. Default is True.
html_show_copyright = True


# -- Options for LaTeX output ---------------------------------------------
# latex_engine = 'pdflatex'

# latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    # 'papersize': 'a4paper',
    # The font size ('10pt', '11pt' or '12pt').
    # 'pointsize': '12pt',
    # Latex figure (float) alignment
    # 'figure_align':'htbp',
    # 'extraclassoptions': 'openany',
    # 'preamble': r'''
        # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        # %%%add number to subsubsection 2=subsection, 3=subsubsection
        # \setcounter{secnumdepth}{0}
        # %%%% Table of content upto 2=subsection, 3=subsubsection
        # \setcounter{tocdepth}{2}
    # ''',

    # 'sphinxsetup': \
        # 'hmargin={0.7in,0.7in}, vmargin={0.7in,0.7in}, \
        # marginpar=1in, \
        # verbatimwithframe=False, \
        # TitleColor={rgb}{0,0,0}, \
        # HeaderFamily=\\rmfamily\\bfseries, \
        # InnerLinkColor={rgb}{0,0,1}, \
        # OuterLinkColor={rgb}{0,0,1}',
# }

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).

# latex_documents = [
  # ('index', 'CulebrONT.tex', 'Documentation',
   # latex_authors, 'manual', True),
# ]

# The name of an image file (relative to this directory) to place at the top of
# the title page.
# latex_logo = '_images/culebront_logo.png'

# For "manual" documents, if this is true, then toplevel headings are parts,
# not chapters.
# latex_use_parts = False

# If true, show page references after internal links.
#latex_show_pagerefs = False

# If true, show URL addresses after external links.
#latex_show_urls = False

# Documents to append as an appendix to all manuals.
#latex_appendices = []

# If false, no module index is generated.
#latex_domain_indices = True

# latex_toplevel_sectioning = 'section'



import pandas as pd
import argparse
from pathlib import Path
from collections import OrderedDict
from pprint import pprint as pp
import re
version = "0.0.1"

def replace_all(repls, str):
    """
    Function that take a dictionary and text variable and return text variable with replace 'Key' from dictionary with 'Value'.

    :param repls: a python dictionary
    :type repls: dict()
    :param str: a string where remplace some words
    :type str: str()
    :rtype: str()
    :return: - txt with replace 'Key' of dictionary with 'Value' in the input txt

    Example:
        >>> text =  "i like apples, but pears scare me"
        >>> print(replace_all({"apple": "pear", "pear": "apple"}, text))
        i like pears, but apples scare me
    """
    import re
    return re.sub('|'.join(re.escape(key) for key in repls.keys()), lambda k: repls[k.group(0)], str)


class AutoVivification(OrderedDict):
    """
    Implementation of perl's autovivification feature.

    Example:

    >>> a = AutoVivification()
    >>> a[1][2][3] = 4
    >>> a[1][3][3] = 5
    >>> a[1][2]['test'] = 6
    >>> print a
    >>> {1: {2: {'test': 6, 3: 4}, 3: {3: 5}}}

    """

    def __getitem__(self, item):
        try:
            return dict.__getitem__(self, item)
        except KeyError:
            value = self[item] = type(self)()
            return value

def main():
    
    assembly_list = snakemake.params.assembly_list
    quality_list = snakemake.params.quality_list
    out_repository = snakemake.params.out_dir
    out_stats = snakemake.output.stat
    summary_flagstats = snakemake.input.summary
    
    dico_flagstats_time = AutoVivification()


    for flagstats_file in sorted(summary_flagstats):
        flagstats_file_path = Path(flagstats_file)
        assembler, step, _ = flagstats_file_path.stem.split("-")
        
        step = replace_all({"STEP_":"","_STARTFIXED":""}, step)
        with open(flagstats_file, 'r') as flagstat:
                for line in flagstat:
                    line = line.strip()
                    if not line.startswith('#') or not line.startswith('\n'):
                        # print(line)
                        if 'in total' in line:
                            total = str(re.search("([0-9]+ [\+] [0-9]+) in total", line).groups()).strip('(').strip(')').strip('\',')
                        elif 'secondary' in line:
                            secondary = str(re.search("([0-9]+ [\+] [0-9]+) secondary", line).groups()).strip('(').strip(')').strip('\',')
                        elif 'supplementary' in line:
                            supplementary = str(re.search("([0-9]+ [\+] [0-9]+) supplementary", line).groups()).strip('(').strip(')').strip('\',')
                        elif 'duplicates' in line:
                            duplicates = str(re.search("([0-9]+ [\+] [0-9]+) duplicates", line).groups()).strip('(').strip(')').strip('\',')
                        elif 'mapped (' in line:
                            mapped = str(re.search("([0-9]+ [\+] [0-9]+) mapped", line).groups()).strip('(').strip(')').strip('\',')
                        elif 'mapped (' in line:
                            mapped = str(re.search("([0-9]+ [\+] [0-9]+) mapped", line).groups()).strip('(').strip(')').strip('\',')
                        #elif 'paired' in line:
                        #    paired = str(re.search("([0-9]+ [\+] [0-9]+) paired in sequencing", line).groups()).strip('(').strip(')').strip('\',')


                            #paired = re.search("([0-9]+ [\+] [0-9]+) paired in sequencing", line).groups()
                            #read1 = re.search("([0-9]+ [\+] [0-9]+) read1", line).groups()
                            #read2 = re.search("([0-9]+ [\+] [0-9]+) read2", line).groups()
                            #properly = re.search("([0-9]+ [\+] [0-9]+) properly paired", line).groups()
                            #mate = re.search("([0-9]+ [\+] [0-9]+) with itself and mate mapped", line).groups()
                            #singletons = re.search("([0-9]+ [\+] [0-9]+) singletons", line).groups()
                            #mate_diff_chr = re.search("([0-9]+ [\+] [0-9]+) with mate mapped to a different chr$", line).groups()
                            #mapq5 = re.search("([0-9]+ [\+] [0-9]+) with mate mapped to a different chr (mapQ>=5)", line).groups()


        dico_flagstats_time[assembler][step]['Total'] = f'{total}'
        dico_flagstats_time[assembler][step]['Secondary'] = f'{secondary}'
        dico_flagstats_time[assembler][step]['Supplementary'] = f'{supplementary}'
        dico_flagstats_time[assembler][step]['Duplicates'] = f'{duplicates}'
        dico_flagstats_time[assembler][step]['Mapped'] = f'{mapped}'
        #dico_flagstats_time[assembler][step]['Paired'] = f'{paired}'

    df = pd.DataFrame.from_dict(dico_flagstats_time)
    #df.columns.values[0] = "Assembler"
    #df.columns.values[1] = "Step"
    dataframe_flag = df.T.stack().apply(pd.Series)
    with open(out_stats, "w") as flagstats_out:
        #columns = ['Assembler', "Step", "Total", "Secondary", "Supplementary", "Duplicates","Mapped"]
        #print(f"dataframe_flag:\n{dataframe_flag}\n")
        dataframe_flag.to_csv(flagstats_out, index=True)

if __name__ == '__main__':
    main()



import click
from shutil import copyfile
from culebrONT.global_variable import *
import os


@click.command("run_cluster", short_help='Run workflow on HPC', context_settings=dict(ignore_unknown_options=True, max_content_width=800), no_args_is_help=True)
@click.option('--config', '-c', type=click.Path(exists=True, file_okay=True, readable=True, resolve_path=True), required=True, show_default=True, help='Configuration file for run culebrONT')
@click.option('--clusterconfig', '-cl', default=CULEBRONT_CLUSTER_CONFIG, type=click.Path(exists=True, file_okay=True, readable=True, resolve_path=True), required=False, show_default=True, help='Overwrite profile clusterconfig file for run culebrONT')
# @click.option('--profile', '-p', default=CULEBRONT_PROFILE, type=click.Path(exists=True, dir_okay=True, readable=True, resolve_path=True), required=False, show_default=True, help='Path to snakemake profile for run culebrONT')
# @click.option('--tools', '-t', default=CULEBRONT_TOOLS_PATH, type=click.Path(exists=True, file_okay=True, readable=True, resolve_path=True), required=False, show_default=True, help='Path to tools_path.yaml for run culebrONT')
@click.option('--pdf', '-pdf', is_flag=True, required=False, default=False, show_default=True, help='run snakemake with --dag, --rulegraph and --filegraph')
@click.argument('snakemake_other', nargs=-1, type=click.UNPROCESSED)
# def run_cluster(config, clusterconfig, profile, tools, pdf, snakemake_other):
def run_cluster(config, clusterconfig, pdf, snakemake_other):
    """
    \b
    Run snakemake command line with mandatory parameters.
    SNAKEMAKE_OTHER: append other snakemake command such '--dry-run'

    Example:
        culebrONT run_cluster -c config.yaml --dry-run --jobs 200
    """
    profile = CULEBRONT_PROFILE
    tools = CULEBRONT_TOOLS_PATH
    # get user arguments
    click.secho(f'    Profile file: {profile}', fg='yellow')
    click.secho(f'    Config file: {config}', fg='yellow')

    if clusterconfig != CULEBRONT_CLUSTER_CONFIG.as_posix():
        CULEBRONT_ARGS_CLUSTER_CONFIG.parent.mkdir(parents=True, exist_ok=True)
        copyfile(clusterconfig, CULEBRONT_ARGS_CLUSTER_CONFIG.as_posix())
        clusterconfig = clusterconfig
    elif CULEBRONT_USER_CLUSTER_CONFIG.exists():
        clusterconfig = CULEBRONT_USER_CLUSTER_CONFIG
    else:
        clusterconfig = CULEBRONT_CLUSTER_CONFIG
    cmd_clusterconfig = f"--cluster-config {clusterconfig}"
    click.secho(f'    Cluster config file load: {clusterconfig}', fg='yellow')

    if tools != CULEBRONT_TOOLS_PATH.as_posix():
        CULEBRONT_ARGS_TOOLS_PATH.parent.mkdir(parents=True, exist_ok=True)
        copyfile(tools, CULEBRONT_ARGS_TOOLS_PATH)
    elif CULEBRONT_USER_TOOLS_PATH.exists():
        tools = CULEBRONT_USER_TOOLS_PATH
    click.secho(f'    Tools Path file: {tools}', fg='yellow')

    cmd_snakemake_base = f"snakemake --show-failed-logs -p -s {CULEBRONT_SNAKEFILE} --configfile {config} --profile {profile} {cmd_clusterconfig} {' '.join(snakemake_other)}"
    click.secho(f"    {cmd_snakemake_base}\n", fg='bright_blue')
    # exit()
    os.system(cmd_snakemake_base)

    if pdf:
        dag_cmd_snakemake = f"{cmd_snakemake_base} --dag | dot -Tpdf > schema_pipeline_dag.pdf"
        click.secho(f"    {dag_cmd_snakemake}\n", fg='bright_blue')
        os.system(dag_cmd_snakemake)
        rulegraph_cmd_snakemake = f"{cmd_snakemake_base} --rulegraph | dot -Tpdf > schema_pipeline_global.pdf"
        click.secho(f"    {rulegraph_cmd_snakemake}\n", fg='bright_blue')
        os.system(rulegraph_cmd_snakemake)
        filegraph_cmd_snakemake = f"{cmd_snakemake_base} --filegraph | dot -Tpdf > schema_pipeline_files.pdf"
        click.secho(f"    {filegraph_cmd_snakemake}\n", fg='bright_blue')
        os.system(filegraph_cmd_snakemake)


@click.command("run_local", short_help='Run workflow on local computer (use singularity mandatory)', context_settings=dict(ignore_unknown_options=True, max_content_width=800),
               no_args_is_help=True)
@click.option('--config', '-c', type=click.Path(exists=True, file_okay=True, readable=True, resolve_path=True), required=True, help='Configuration file for run culebrONT')
@click.option('--threads', '-t', type=int, required=True, help='number of threads')
@click.option('--pdf', '-p', is_flag=True, required=False, default=False, help='run snakemake with --dag, --rulegraph and --filegraph')
@click.argument('snakemake_other', nargs=-1, type=click.UNPROCESSED)
def run_local(config, threads, pdf, snakemake_other):
    """    Run snakemake command line with mandatory parameters.
    SNAKEMAKE_OTHER: append other snakemake command such '--dry-run'

    Example:
        culebrONT run_local -c config.yaml --dry-run
    """
    # get user arguments
    click.secho(f'    Config file: {config}', fg='yellow')

    cmd_snakemake_base = f"snakemake --latency-wait 60000 --cores {threads} --use-singularity --singularity-args \'--bind $HOME\' --show-failed-logs --printshellcmds -s {CULEBRONT_SNAKEFILE} --configfile {config}  {' '.join(snakemake_other)}"
    click.secho(f"\n    {cmd_snakemake_base}\n", fg='bright_blue')

    os.system(cmd_snakemake_base)
    if pdf:
        dag_cmd_snakemake = f"{cmd_snakemake_base} --dag | dot -Tpdf > schema_pipeline_dag.pdf"
        click.secho(f"    {dag_cmd_snakemake}\n", fg='bright_blue')
        os.system(dag_cmd_snakemake)
        rulegraph_cmd_snakemake = f"{cmd_snakemake_base} --rulegraph | dot -Tpdf > schema_pipeline_global.pdf"
        click.secho(f"    {rulegraph_cmd_snakemake}\n", fg='bright_blue')
        os.system(rulegraph_cmd_snakemake)
        filegraph_cmd_snakemake = f"{cmd_snakemake_base} --filegraph | dot -Tpdf > schema_pipeline_files.pdf"
        click.secho(f"    {filegraph_cmd_snakemake}\n", fg='bright_blue')
        os.system(filegraph_cmd_snakemake)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import os
from pathlib import Path
from setuptools import setup, find_packages

NAME = 'culebrONT'
URL = 'https://github.com/SouthGreenPlatform/CulebrONT_pipeline'
CURRENT_PATH = Path(__file__).resolve().parent
VERSION = CURRENT_PATH.joinpath(f'{NAME}','VERSION').open('r').readline().strip()

def main():
    setup(
        # Project information
        name=NAME,
        version=VERSION,
        url=URL,
        project_urls={
            'Bug Tracker': f'{URL}/issues',
            'Documentation': f'https://{NAME}.readthedocs.io/en/latest/',
            'Source Code': URL
        },
        download_url=f'{URL}/archive/{VERSION}.tar.gz',
        author='''Julie Orjuela (IRD),
                   Aurore Comte (IRD),
                   Sebastien Ravel (CIRAD),
                   Florian Charriat (INRAE),
                   Bao Tram Vi (IRD),
                   François Sabot (IRD)
                   Sebastien Cunnac (IRD)''',
        author_email='sebastien.ravel@cirad.fr',
        #description=culebrONT.__doc__,
        long_description=CURRENT_PATH.joinpath('README.rst').open('r', encoding='utf-8').read(),
        long_description_content_type='text/x-rst',
        license='GPLv3',

        # docs compilation utils
        command_options={
            'build_sphinx': {
                'project': ('setup.py', NAME),
                'version': ('setup.py', VERSION),
                'release': ('setup.py', VERSION),
                'source_dir': ('setup.py', CURRENT_PATH.joinpath('docs','source').as_posix()),
                'build_dir': ('setup.py', CURRENT_PATH.joinpath('docs','build').as_posix()),
            }},

        # Package information
        packages=find_packages(),
        package_data={
            '': ['*'],
        },
        include_package_data=True,
        python_requires='>=3.6',
        install_requires=[
            'PyYAML',
            'pandas',
            'matplotlib',
            'tabulate',
            'rpy2',
            'ipython',
            'biopython',
            'pysam',
            'numpy',
            'argparse',
            'snakemake',
            'tqdm',
            'click>=8.0.3',
            'cookiecutter'
        ],
        extras_require={
            'docs': ['sphinx_copybutton',
                     'sphinx_rtd_theme',
                     'sphinx_click'],
            'dev': ['tox'],
        },
        entry_points={
            'culebrONT': ['culebrONT = __init__'],
            'console_scripts': [f'culebrONT = culebrONT.main:main']},

        # Pypi information
        platforms=['unix', 'linux'],
        keywords=[
            'snakemake',
            'assembly',
            'workflow'
        ],
        classifiers=[
            'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
            'License :: CeCILL-C Free Software License Agreement (CECILL-C)',
            'License :: Free for non-commercial use',
            'Development Status :: 5 - Production/Stable',
            'Intended Audience :: Developers',
            'Intended Audience :: End Users/Desktop',
            'Operating System :: POSIX :: Linux',
            'Programming Language :: Python :: 3.6',
            'Programming Language :: Python :: 3.7',
            'Programming Language :: Python :: 3.8',
            'Programming Language :: Python :: 3.9',
            'Programming Language :: R',
            'Natural Language :: English',
            'Topic :: Scientific/Engineering',
            'Topic :: Scientific/Engineering :: Bio-Informatics',
            'Topic :: Software Development :: Assemblers'
        ],
        options={
            'bdist_wheel': {'universal': False}
        },
        zip_safe=False,  # Don't install the lib as an .egg zipfile
    )


if __name__ == '__main__':
    main()
